/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "caosdb/data_type.h"               // for DataType, AtomicDataType
#include "caosdb/entity.h"                  // for Entity
#include "caosdb/entity/v1alpha1/main.pb.h" // for AtomicDataType, DataType
#include "caosdb/value.h"                   // for Value
#include <gtest/gtest-message.h>            // for Message
#include <gtest/gtest-test-part.h>          // for TestPartResult, SuiteApi...
#include <gtest/gtest_pred_impl.h>          // for AssertionResult, Test
#include <memory>                           // for allocator_traits<>::valu...
#include <string>                           // for string
#include <vector>                           // for vector

namespace caosdb::entity {
using ProtoEntity = caosdb::entity::v1alpha1::Entity;
using ProtoParent = caosdb::entity::v1alpha1::Parent;
using ProtoDataType = caosdb::entity::v1alpha1::DataType;
using ProtoAtomicDataType = caosdb::entity::v1alpha1::AtomicDataType;

TEST(test_list_property, test_list_of_text) {
  Property list_property;
  list_property.SetDataType(DataType::ListOf(AtomicDataType::TEXT));
  std::vector<std::string> in_value{"item1", "item2", "item3"};
  list_property.SetValue(in_value);

  Entity entity;
  entity.SetRole(Role::RECORD_TYPE);
  entity.SetName("TestRT");
  entity.AppendProperty(list_property);

  const auto &data_type = entity.GetProperties().at(0).GetDataType();
  const auto &value = entity.GetProperties().at(0).GetValue();

  EXPECT_TRUE(data_type.IsList());
  EXPECT_TRUE(data_type.AsList().IsListOfAtomic());
  EXPECT_EQ(data_type.AsList().GetAtomicDataType(), AtomicDataType::TEXT);

  EXPECT_TRUE(value.IsList());
  EXPECT_EQ(value.AsList().size(), 3);
  EXPECT_TRUE(value.AsList().at(1).IsString());
  EXPECT_EQ(value.AsList().at(1).AsString(), "item2");
}

} // namespace caosdb::entity
